from optparse import OptionParser

from parsley.config import *
from parsley.loggingex import *

from util import *

log = logging.getLogger("cumin.config")

class CuminConfig(Config):
    def __init__(self):
        super(CuminConfig, self).__init__()

        hdef = os.path.normpath("/usr/share/cumin")
        self.home = os.environ.get("CUMIN_HOME", hdef)

        if not os.path.isdir(self.home):
            raise Exception("Home path '%s' is not a directory")

        web = CuminConfigSection(self, "web")
        web.log_file.default = os.path.join(self.home, "log", "web.log")

        param = ConfigParameter(web, "update-interval", int)
        param.default = 10

        param = ConfigParameter(web, "host", str)
        param.default = "localhost"

        param = ConfigParameter(web, "port", int)
        param.default = 45672

        param = ConfigParameter(web, "operator-email", str)

        param = ConfigParameter(web, "user", str)

        data = CuminConfigSection(self, "data")
        data.log_file.default = os.path.join(self.home, "log", "data.log")
        
        param = ConfigParameter(data, "packages", str)

        param = ConfigParameter(data, "expire-interval", int)
        param.default = 60 * 60 # 1 hour

        param = ConfigParameter(data, "expire-threshold", int)
        param.default = 24 * 60 * 60 # 1 day

        param = ConfigParameter(data, "vacuum-interval", int)
        param.default = 60 * 60 # 1 hour

    def parse(self):
        paths = list()

        paths.append(os.path.join(os.sep, "etc", "cumin", "cumin.conf"))
        paths.append(os.path.join(self.home, "etc", "cumin.conf"))
        paths.append(os.path.join(os.path.expanduser("~"), ".cumin.conf"))

        return self.parse_files(paths)

class CuminConfigSection(ConfigSection):
    def __init__(self, config, name):
        super(CuminConfigSection, self).__init__(config, name)

        param = ConfigParameter(self, "database", str)
        param.default = "dbname=cumin user=cumin host=localhost"

        param = ConfigParameter(self, "brokers", str)
        param.default = "amqp://localhost"

        self.log_file = ConfigParameter(self, "log-file", str)

        param = ConfigParameter(self, "log-level", str)
        param.default = "info"

        param = ConfigParameter(self, "debug", bool)
        param.default = False

class CuminOptionParser(OptionParser):
    def __init__(self, section):
        OptionParser.__init__(self)

        self.add_option("--database", default=section.database)
        self.add_option("--brokers", default=section.brokers)
        self.add_option("--log-file", default=section.log_file)
        self.add_option("--log-level", default=section.log_level)
        self.add_option("--debug", action="store_true", default=section.debug)
        self.add_option("--init-only", action="store_true")

def setup_logging(values):
    modules = ("cumin", "mint", "parsley", "rosemary", "wooly")

    for name in modules:
        enable_logging(name, values.log_level, values.log_file)

    if values.debug:
        for name in modules:
            enable_logging(name, "debug", sys.stderr)
