import logging

from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *

from cumin.objectframe import *
from cumin.objecttask import *
from cumin.stat import *
from cumin.widgets import *
from cumin.parameters import *
from cumin.formats import *
from cumin.util import *

import main

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.grid.collector")

class CollectorFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector

        super(CollectorFrame, self).__init__(app, name, cls)

        self.view.add_tab(CollectorOverview(app, "stats", self.object))

        self.start = DaemonStart(app, self, "COLLECTOR")
        self.stop = DaemonStop(app, self, "COLLECTOR")

class CollectorGeneralStatSet(StatSet):
    def __init__(self, app, name, object):
        super(CollectorGeneralStatSet, self).__init__(app, name, object)

        self.attrs = ("RunningJobs", "IdleJobs",
                      "HostsClaimed", "HostsUnclaimed",
                      "HostsOwner", "HostsTotal")

class CollectorOverview(Widget):
    def __init__(self, app, name, collector):
        super(CollectorOverview, self).__init__(app, name)

        self.add_child(GeneralStatSet(app, "stats", collector))

        chart = self.JobStackedChart(app, "jobs", collector)
        chart.stats = ("RunningJobs", "IdleJobs")
        chart.chart_type = "stacked"
        self.add_child(chart)

        chart = self.SlotStackedChart(app, "slots", collector)
        chart.stats = ("HostsClaimed", "HostsUnclaimed", "HostsOwner")
        chart.chart_type = "stacked"
        self.add_child(chart)

    def render_title(self, session):
        return "Overview"

    class JobStackedChart(StatFlashChart):
        def render_title(self, session):
            return "Job status"

    class SlotStackedChart(StatFlashChart):
        def render_title(self, session, *args):
            return "Slot state"
