import logging

from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *
from time import time
from cumin.widgets import *
from cumin.model import FetchJobAd, FetchJobOutput
from cumin.objecttask import *
from cumin.parameters import *
from cumin.stat import *
from cumin.formats import *
from cumin.util import *
from cumin.qmfadapter import *
from parsley.stringex import partition

import main

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.job")

class JobFrame(ObjectFrame):
    def __init__(self, app, name, submission):
        cls = app.model.com_redhat_grid.Submission

        super(JobFrame, self).__init__(app, name, cls)

        self.view = JobView(app, "view", self.object)
        self.replace_child(self.view)

        # view or edit
        self.ads = JobAdModes(app, "ads")
        self.view.add_tab(self.ads)

        self.view.add_tab(JobOutput(app, "output"))

        self.job_id = Parameter(app, "job_id")
        self.add_parameter(self.job_id)

        JobHold(app, self)
        JobRelease(app, self)
        JobRemove(app, self)

        # task not visible on page. used for edit submit
        self.set_ad_task = JobSetAttribute(app, self)

    def get_title(self, session):
        job_id = self.job_id.get(session)
        return "Job %s" % job_id

    def show_ads_edit(self, session):
        self.ads.editor.show(session)

    def show_ads_view(self, session):
        self.ads.viewer.show(session)

    def get_submission(self, session, id):
        return self.get_object(session, id)

    def get_job_server(self, session, id):
        submission = self.get_submission(session, id)

        cls = self.app.model.com_redhat_grid.JobServer
        return cls.get_object(session.cursor, _id=submission._jobserverRef_id)

    def get_scheduler(self, session, id):
        job_server = self.get_job_server(session, id)

        cls = self.app.model.com_redhat_grid.Scheduler
        return cls.get_object(session.cursor, _id=job_server._schedulerRef_id)

    def get_href(self, session, id, job_id):
        branch = session.branch()

        self.id.set(branch, id)
        self.job_id.set(branch, job_id)
        self.view.show(branch)

        return branch.marshal()

class JobAdModes(ModeSet):
    def __init__(self, app, name):
        super(JobAdModes, self).__init__(app, name)

        self.viewer = JobAdsViewer(app, "viewer")
        self.add_mode(self.viewer)

        self.editor = JobAdsEditor(app, "editor")
        self.add_mode(self.editor)

    def render_title(self, session):
        return "Attributes"

class JobView(ObjectView):
    def add_details_tab(self):
        pass

    def render_title(self, session):
        return self.frame.get_title(session)

class JobSummariesAdapter(ObjectQmfAdapter):
    def get_data(self, values, options):
        first_index = options.offset
        last_index = options.offset + options.limit

        submission = values["obj"]
        results = self.app.model.get_submission_job_summaries(submission)

        if isinstance(results.data, dict):
            rows = self.process_results(results.data)
        elif isinstance(results.data, list):
            rows = self.process_list_results(results.data)
        else:
            rows = []
        page = rows[first_index:last_index]

        return page, results

    def process_list_results(self, results):
        # the results for this call is a list
        records = list()

        if results:
            for rec in results:
                row = self.process_record(None, rec)
                records.append(row)

        return records

    def process_record(self, key, record):
        field_data = list()
        for column in self.columns:
            try:
                val = record[column.name]
            except KeyError:
                val = None
                if column.name == "JobId":
                    # GlobalJobId should look like localhost6.localdomain6#94.1#1284091602
                    try:
                        gjid = record["GlobalJobId"]
                        parts = gjid.split("#")
                        if len(parts) == 3:
                            val = parts[1]
                    except Exception, e:
                        pass
                    if not val:
                        try:
                            val = "%s.%s" % (str(record["ClusterId"]), str(record["ProcId"]))
                        except KeyError:
                            val = "0"
                else:
                    val = 0
            field_data.append(val)
        return field_data

    def get_count(self, values):
        submission = values["obj"]
        results = self.app.model.get_submission_job_summaries(submission)
        return results.data and len(results.data) or 0

class NonSortableObjectTableColumn(ObjectTableColumn):
    def __init__(self, app, name, attr):
        super(NonSortableObjectTableColumn, self).__init__(app, name, attr)

        self.header = StaticColumnHeader(app, "header")
        self.replace_child(self.header)

class NonSortableObjectLinkColumn(ObjectLinkColumn):
    def __init__(self, app, name, attr, id_attr, frame_path):
        super(NonSortableObjectLinkColumn, self).__init__(app, name, attr, id_attr, frame_path)

        self.header = StaticColumnHeader(app, "header")
        self.replace_child(self.header)

class JobSelector(ObjectSelector):
    def __init__(self, app, name, submission):
        cls = app.model.com_redhat_cumin_grid.JobSummary

        super(JobSelector, self).__init__(app, name, cls)

        self.adapter = JobSummariesAdapter(app, cls)

        self.submission = submission
        frame = "main.grid.pool.submission.job"
        self.job_id_col = self.JobIdColumn(app, "job", cls.GlobalJobId, cls.JobId, frame)
        self.add_column(self.job_id_col)

        status_column = self.Status(app, "status", cls.JobStatus)
        self.add_column(status_column)

        cmd_column = NonSortableObjectTableColumn(app, cls.Cmd.name, cls.Cmd)
        self.add_column(cmd_column)

        self.job_id_column = ObjectTableColumn(app, cls.JobId.name, cls.JobId)
        self.job_id_column.visible = False
        self.add_column(self.job_id_column)

        self.error_tmpl = WidgetTemplate(self, "error_html")
        self.qmf_response = Attribute(app, "qmf_response")
        self.add_attribute(self.qmf_response)

        JobSelectionHold(app, self, "held")
        JobSelectionRelease(app, self, "released")
        JobSelectionRemove(app, self, "removed")

    def init_ids(self, app, cls):
        item = Parameter(app, "item")

        self.ids = ListParameter(app, "selection", item)
        self.add_parameter(self.ids)

        self.checkbox_column = ObjectCheckboxColumn \
            (app, "id", cls.JobId, self.ids)
        self.add_column(self.checkbox_column)

    def render_title(self, session):
        return "Jobs"

    def do_render(self, session):
        # determine if there is an error getting the data
        # and show an error message
        self.get_data(session)
        qmf_response = self.qmf_response.get(session)
        if qmf_response.exception:
            writer = Writer()
            msg = "The data is not available at this time. <br/>%s" % str(qmf_response.exception.message)
            self.error_tmpl.render(writer, session, msg)
            return writer.to_string()

        return super(JobSelector, self).do_render(session)

    def get_data(self, session):
        values = self.get_data_values(session)
        options = self.get_data_options(session)

        results, qmf_response = self.adapter.get_data(values, options)
        self.qmf_response.set(session, qmf_response)
        return results

    def render_error_msg(self, session, msg):
        return msg

    def get_data_values(self, session):
        values = super(JobSelector, self).get_data_values(session)

        submission = self.submission.get(session)

        if submission:
            values['obj'] = submission
            values['args'] = ()
        return values

    class JobIdColumn(NonSortableObjectLinkColumn):
        def render_cell_href(self, session, record):
            job_id = record[self.parent.job_id_column.field.index]
            #frame = self.page.page_widgets_by_path[self.frame_path]
            frame = self.table.frame.job

            submission = self.parent.submission.get(session)
            return frame.get_href(session, submission._id, job_id)

    class Status(NonSortableObjectTableColumn):
        def render_cell_content(self, session, record):
            status = self.field.get_content(session, record)
            return JobStatusInfo.get_status_string(status)

class JobObjectSelectorTask(ObjectSelectorTask):
    def __init__(self, app, selector, verb):
        super(JobObjectSelectorTask, self).__init__(app, selector)

        self.form = JobObjectSelectorTaskForm(app, self.name, self, verb)
        self.method = None

    def do_invoke(self, invoc, job_id, scheduler, reason):
        assert self.method

        action = QmfCall(self.app)
        results = action.execute(scheduler, self.method, job_id, reason)

        if results.error:
            raise results.error

        invoc.status_code = results.status
        invoc.end()

    def do_enter(self, session, osession):
        submission = self.selector.submission.get(osession)
        self.form.submission_id.set(session, submission._id)

class JobSelectionHold(JobObjectSelectorTask):
    def __init__(self, app, selector, verb):
        super(JobSelectionHold, self).__init__(app, selector, verb)

        self.method = "HoldJob"

    def get_title(self, session):
        return "Hold"

class JobSelectionRelease(JobObjectSelectorTask):
    def __init__(self, app, selector, verb):
        super(JobSelectionRelease, self).__init__(app, selector, verb)

        self.method = "ReleaseJob"

    def get_title(self, session):
        return "Release"

class JobSelectionRemove(JobObjectSelectorTask):
    def __init__(self, app, selector, verb):
        super(JobSelectionRemove, self).__init__(app, selector, verb)

        self.method = "RemoveJob"

    def get_title(self, session):
        return "Remove"

class JobObjectSelectorTaskForm(ObjectSelectorTaskForm):
    def __init__(self, app, name, task, verb):
        super(JobObjectSelectorTaskForm, self).__init__(app, name, task)

        self.submission_id = IntegerParameter(app, "sub")
        self.add_parameter(self.submission_id)

        self.content = ReasonField(app, "fields")
        self.content.required = True
        self.replace_child(self.content)

        self.verb = verb

    def get_selection(self, session):
        ids = self.ids.get(session)

        selection = list(ids)
        self.selection.set(session, selection)

    def get_reason(self, session, verb):
        """ returns <verb> by username[: <user input reason>] """
        reason = self.content.get(session)
        if reason:
            reason = [reason]
            user = session.client_session.attributes["login_session"].user

            verb_by = "%s by %s" % (verb, user.name)
            reason.insert(0, verb_by)
            return ": ".join(reason)

    def process_submit(self, session):
        selection = self.selection.get(session)
        reason = self.get_reason(session, self.verb)

        scheduler = self.get_scheduler(session)

        self.task.invoke(session, selection, scheduler, reason)
        self.task.exit_with_redirect(session)

    def get_scheduler(self, session):
        submission_id = self.submission_id.get(session)
        cls = self.app.model.com_redhat_grid.Submission
        submission = cls.get_object_by_id(session.cursor, submission_id)

        cls = self.app.model.com_redhat_grid.JobServer
        job_server = cls.get_object(session.cursor, _id=submission._jobserverRef_id)

        cls = self.app.model.com_redhat_grid.Scheduler
        return cls.get_object(session.cursor, _id=job_server._schedulerRef_id)

class JobAdsSet(PropertySet):
    types = {0: "expression",
             1: "integer",
             2: "float",
             3: "string"}
    def __init__(self, app, name):
        super(JobAdsSet, self).__init__(app, name)

        self.items = Attribute(app, "cached_items")
        self.add_attribute(self.items)

        self.qmf_error = Attribute(app, "qmf_error")
        self.add_attribute(self.qmf_error)

    def do_get_items(self, session):
        ad_list = self.items.get(session)
        error = self.qmf_error.get(session)

        if not ad_list and not error:
            ad_list = list()
            id = self.frame.id.get(session)
            job_server = self.frame.get_job_server(session, id)
            job_id = self.frame.job_id.get(session)

            action = QmfCall(self.app, {'JobAd': {}})
            results = action.execute(job_server, "GetJobAd", job_id)
            error = results.error
            self.qmf_error.set(session, error)
            ads = results.data['JobAd']
            cls = self.app.model.job_meta_data
            ad_list = [self.gen_item(x, ads[x], cls, dtype=self.get_type(ads[x])) \
                       for x in ads if not x.startswith("!!")]

            self.items.set(session, ad_list)

        return ad_list, error

    def get_type(self, value):
        if isinstance(value, (int, long)):
            type = "integer"
        elif isinstance(value, float):
            type = "float"
        elif isinstance(value, dict):
            type = "dict"
        else:
            type = "string"
        return type

    def gen_item(self, name, value, cls, path=None, dtype=None,
                 error=None, orig=None):
        """ Generate a dict with name, value, type, error, path,
        property, orig

            This is called with raw GetAd data and with processed data
            from a form submit. With raw data, only the name and value
            will be present.  With form data, we might have a path,
            dtype, or error. dtype is the data type that was
            remembered from the raw data.
        """

        idict = dict()
        idict["name"] = name
        idict["value"] = value
        idict["orig"] = value
        idict["type"] = dtype

        if dtype == "string":
            idict["value"] = strip_string_quotes(value)

        if orig:
            idict["orig"] = orig

        if error and "error" in error:
            idict["error"] = error["error"]

        if name in cls.ad_properties_by_name:
            idict["property"] = cls.ad_properties_by_name[name]

        if path:
            idict["path"] = path

        return idict

class JobPropertyRenderer(TemplateRenderer):
    def render_title(self, session, item):
        title = item["name"]
        if "property" in item:
            property = item["property"]
            if property.title:
                title = property.get_title(session)
        return escape_amp(title)

    def render_value(self, session, item):
        value = item["value"]
        if "property" in item:
            property = item["property"]
            if property.renderer:
                value = property.renderer(session, value)
        if item["type"] == "dict":
            return value
        else:
            ret = escape_entity(str(value))
            return self.insert_breaks(ret)

    def insert_breaks(self, value):
        subwords = list()
        snippets = parse(value, begin_delim="&", end_delim=";")
        for snippet in snippets:
            while len(snippet) > 40:
                subwords.append(snippet[:40])
                snippet = snippet[40:]
            subwords.append(snippet)

        return "&#8203;".join(subwords)

    def render_inline_help(self, session, item):
        if "property" in item:
            property = item["property"]
            return property.description

class JobAdsGroups(Widget):
    def __init__(self, app, name):
        super(JobAdsGroups, self).__init__(app, name)

        self.group_tmpl = WidgetTemplate(self, "group_html")
        self.error_tmpl = WidgetTemplate(self, "error_html")

    def render_groups(self, session):
        writer = Writer()

        # check for qmf error
        _, error = self.parent.do_get_items(session)
        if error:
            msg = error.message
            if not msg:
                # probably a timeout exception
                msg = "Unable to get Job information at this time"
            self.error_tmpl.render(writer, session, msg)
        else:
            for group in self.app.model.get_ad_groups():
                self.group_tmpl.render(writer, session, group)

        return writer.to_string()

    def render_group_name(self, session, group):
        return group

    def render_properties(self, session, group):
        items, error = self.parent.get_group_items(session, group)
        writer = Writer()

        for item in items:
            self.parent.item_renderer.render(writer, session, item)

        return writer.to_string()

    def render_error_msg(self, session, msg):
        return msg

class JobAdsViewer(JobAdsSet):
    def __init__(self, app, name):
        super(JobAdsViewer, self).__init__(app, name)

        self.item_renderer = JobPropertyRenderer(self, "property_html")

        self.groups = JobAdsGroups(app, "groups");
        self.add_child(self.groups)

        self.wait = Wait(app, "wait")
        self.add_child(self.wait)

        self.edit_button = JobAdsEditButton(app, "edit_button")
        self.add_child(self.edit_button)

        self.defer_enabled = True

    def render_edit_button(self, session):
        _, error = self.do_get_items(session)
        return not error and self.edit_button.render(session) or ""

    def render_title(self, session):
        return "Attributes"

    def get_group_items(self, session, group):
        group_items = list()

        items, error = self.do_get_items(session)
        for item in items:
            if "property" in item:
                property = item["property"]
                item_group = property.group
            else:
                item_group = "Other"
            if item_group == group:
                group_items.append(item)

        return group_items, error

class JobAdsEditButton(Widget):
    def render_edit_ads_url(self, session):
        branch = session.branch()
        self.parent.frame.show_ads_edit(branch)
        return branch.marshal()

class JobAdsEditor(JobAdsViewer, CuminForm):
    def __init__(self, app, name):
        super(JobAdsEditor, self).__init__(app, name)

        # the parameter that will hold all the field values
        self.ads = DictParameter(app, "params")
        self.add_parameter(self.ads)

        self.item_renderer = EditablePropertyRenderer(self, "property_html")

    def do_get_items(self, session):
        ads = self.ads.get(session)

        if len(ads):
            # we just returned from a submit
            cls = self.app.model.job_meta_data
            return [self.gen_item(x, ads[x]["value"], cls, path=self.ads.path,
                                  dtype=ads[x]["type"], error=ads[x],
                                  orig=ads[x]["orig"]) for x in ads]

        items, error = super(JobAdsEditor, self).do_get_items(session)
        for item in items:
            item["path"] = self.ads.path
        return items, error

    def process_cancel(self, session):
        branch = session.branch()
        self.ads.set(branch, None) # otherwise url is too long
        self.frame.show_ads_view(branch)
        self.page.redirect.set(session, branch.marshal())

    def process_submit(self, session):
        ads = self.ads.get(session)
        errors = False

        just_ads = dict()
        for field in ads:
            try:
                fval = ads[field]["value"]
            except KeyError:
                ads[field]["value"] = ""
                fval = ""
            ftype = ads[field]["type"]
            if ftype == "integer":
                try:
                    fval = int(fval)
                except:
                    ads[field]["error"] = "Integer value expected"
                    errors = True
            elif ftype == "float":
                try:
                    fval = float(fval)
                except:
                    ads[field]["error"] = "Floating point value expected"
                    errors = True
            else:
                fval = unicode(fval)

            if "orig" in ads[field]:
                orig = ads[field]["orig"]
                if ftype == "integer":
                    orig = int(orig)
                elif ftype == "float":
                    orig = float(orig)
                if fval != orig:
                    if ftype == "string":
                        quoted = "\"%s\"" % fval
                        if quoted == orig:
                            continue
                    just_ads[unicode(field)] = fval

        if not errors:
            id = self.frame.id.get(session)
            scheduler = self.frame.get_scheduler(session, id)
            job_id = self.frame.job_id.get(session)

            task = self.frame.set_ad_task
            for field in just_ads:
                task.invoke(session, scheduler, job_id, field, str(just_ads[field]))
            self.process_cancel(session)

class OutputFile(Widget):
    err_msg = "Output, Error, and UserLog file names are invalid."
    def __init__(self, app, name, which_file, first_last):
        super(OutputFile, self).__init__(app, name)

        self.which_file = which_file
        self.first_last = first_last

        self.defer_enabled = True

    def render_content(self, session):
        id = self.frame.id.get(session)
        job_server = self.frame.get_job_server(session, id)
        job_id = self.frame.job_id.get(session)
        file, start, end = self.get_file_args(session)
        if file:
            action = QmfCall(self.app, {'Data': ""})
            result = action.execute(job_server, "FetchJobData", job_id, file, start, end)
            if result.error:
                return result.status
            return escape_entity(result.data['Data'])
        return self.err_msg

    def get_file_args(self, session):
        first_last = self.first_last.get(session)
        if first_last == "t":
            start = -2048
            end = 0
        else:
            start = 0
            end = 2048
        file = self.which_file.get_current_file_name(session)
        return (file, start, end)

    def render_loading(self, session, *args):
        file = self.which_file.get_current_file_name(session)
        return file and "loading..." or self.err_msg

class JobOutput(JobAdsSet, Form):
    def __init__(self, app, name):
        super(JobOutput, self).__init__(app, name)

        self.which_file = self.FileSwitch(app, "file")
        self.add_child(self.which_file)

        self.first_last = self.FLSwitch(app, "first_last")
        self.add_child(self.first_last)

        self.__fetch = self.FetchButton(app, "refresh")
        self.add_child(self.__fetch)

        self.output = OutputFile(app, "job_output", self.which_file, self.first_last)
        self.add_child(self.output)

    def render_title(self, session):
        return "Output"

    def render_is_tail(self, session):
        tail = self.first_last.get(session)
        return tail == "t" and "1" or "0"

    def render_out_time(self, session):
        now = datetime.now()
        return fmt_datetime(now, sec=True)

    def do_process(self, session):
        out_file = None
        user_file = None
        err_file = None

        id = self.frame.id.get(session)
        scheduler = self.frame.get_scheduler(session, id)
        ads, error = self.do_get_items(session)
        for ad in ads:
            if ad['name'] == "Out":
                out_file = ad['value']
            elif ad['name'] == "UserLog":
                user_file = ad['value']
            elif ad['name'] == "Err":
                err_file = ad['value']

        # set title for radiotab so mouseover will display file name
        self.which_file.set_file_name(session, "o", out_file)
        self.which_file.set_file_name(session,"e", err_file)
        self.which_file.set_file_name(session,"u", user_file)

        if self.which_file.is_bad(out_file):
            self.which_file.disable(session, "o")
        if self.which_file.is_bad(err_file):
            self.which_file.disable(session, "e")
        if self.which_file.is_bad(user_file):
            self.which_file.disable(session, "u")

        return super(JobOutput, self).do_process(session)

    class FetchButton(FormButton):
        def render_content(self, session):
            return "Refresh"

    class FileSwitch(StateSwitch):
        def __init__(self, app, name):
            super(JobOutput.FileSwitch, self).__init__(app, name)

            self.add_state("o", "Output")
            self.add_state("e", "Error")
            self.add_state("u", "UserLog")

            self.disabled = self.DisabledList(app, "disabled")
            self.add_attribute(self.disabled)

            self.link_titles = self.Titles(app, "link_titles")
            self.add_attribute(self.link_titles)

        class DisabledList(Attribute):
            def get_default(self, session):
                return list()

        class Titles(Attribute):
            def get_default(self, session):
                return dict()

        def disable(self, session, state):
            disabled = self.disabled.get(session)
            disabled.append(state)
            self.disabled.set(session, disabled)
            if state == self.get(session):
                self.select_first_enabled(session)

        def set_file_name(self, session, state, link_title):
            link_titles = self.link_titles.get(session)
            link_titles[state] = link_title
            self.link_titles.set(session, link_titles)

        def get_file_name(self, session, state):
            link_titles = self.link_titles.get(session)
            return state in link_titles and link_titles[state] or ""

        def get_current_file_name(self, session):
            state = self.get(session)
            return self.get_file_name(session, state)

        def select_first_enabled(self, session):
            states = self.get_items(session)
            disabled = self.disabled.get(session)
            for state in states:
                if not state in disabled:
                    self.set(session, state)
                    break

        def render_item_link(self, session, state):
            branch = session.branch()
            self.set(branch, state)

            title = self.get_title(state)
            link_titles = self.link_titles.get(session)
            link_title = state in link_titles and link_titles[state] or ""
            disabled = self.disabled.get(session)
            if state in disabled:
                class_ = "disabled"
                href = "javascript:void(0)"
            else:
                class_ = self.get(session) == state and "selected"
                href = branch.marshal()
            return fmt_link(href, title, class_, link_title=link_title)

        def is_bad(self, file):
            bad = False
            if not file:
                bad = True
            elif "/dev/null" in file.lower():
                bad = True
            return bad


    class FLSwitch(StateSwitch):
        def __init__(self, app, name):
            super(JobOutput.FLSwitch, self).__init__(app, name)

            self.add_state("t", "Tail", "Display end of file")
            self.add_state("h", "Head", "Display beginning of file")

class JobAction(ObjectFrameTask):
    def __init__(self, app, frame, verb):
        super(JobAction, self).__init__(app, frame)

        self.form = JobActionForm(app, self.name, self, verb)

        self.method = None

    def do_enter(self, session, osession):
        job_id = self.frame.job_id.get(osession)
        self.form.job_id.set(session, job_id)

    def do_invoke(self, invoc, scheduler, job_id, reason):
        assert self.method

        action = QmfCall(self.app)
        results = action.execute(scheduler, self.method, job_id, reason)

        if results.error:
            raise results.error
        invoc.status_code = results.status
        invoc.end()

class JobActionForm(ObjectFrameTaskForm):
    def __init__(self, app, name, task, verb):
        super(JobActionForm, self).__init__(app, name, task)

        self.reason = ReasonField(app, "reason")
        self.reason.required = True
        self.add_field(self.reason)

        self.job_id = Parameter(app, "job_id")
        self.add_parameter(self.job_id)

        self.verb = verb

    def get_reason(self, session, verb):
        """ returns <verb> by username[: <user input reason>] """
        reason = self.reason.get(session)
        if reason:
            reason = [reason]
            user = session.client_session.attributes["login_session"].user

            verb_by = "%s by %s" % (verb, user.name)
            reason.insert(0, verb_by)
            return ": ".join(reason)

    def process_submit(self, session):
        self.validate(session)

        if not self.errors.get(session):
            id = self.id.get(session)
            scheduler = self.task.frame.get_scheduler(session, id)
            reason = self.get_reason(session, self.verb)
            job_id = self.job_id.get(session)

            self.task.invoke(session, scheduler, job_id, reason)
            self.task.exit_with_redirect(session)

class ReasonField(StringField):
    def render_title(self, session):
        return "Reason"

class JobHold(JobAction):
    def __init__(self, app, frame):
        super(JobHold, self).__init__(app, frame, "held")

        self.method = "HoldJob"

    def get_title(self, session):
        return "Hold Job"


class JobRelease(JobAction):
    def __init__(self, app, frame):
        super(JobRelease, self).__init__(app, frame, "held")

        self.method = "ReleaseJob"

    def get_title(self, session):
        return "Release Job"

class JobRemove(JobAction):
    def __init__(self, app, frame):
        super(JobRemove, self).__init__(app, frame, "removed")

        self.method = "RemoveJob"

    def get_title(self, session):
        return "Remove Job"

class JobSetAttribute(ObjectFrameTask):
    def get_title(self, session):
        pass

    def get_description(self, session):
        return "Edit Ad"

    def do_invoke(self, invoc, scheduler, job_id, name, value):
        self.qmf_call(invoc, scheduler, "SetJobAttribute", job_id, name, value)

