from wooly import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *
from wooly.widgets import *

from cumin.formats import *
from cumin.model import FetchRawConfigSet
from cumin.objectframe import *
from cumin.objectselector import *
from cumin.parameters import *
from cumin.stat import *
from cumin.util import *
from cumin.widgets import *

from parsley.stringex import partition, rpartition

from daemon import *

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.grid.negotiator")

class NegotiatorFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Negotiator

        super(NegotiatorFrame, self).__init__(app, name, cls)

        #self.edit_dynamic_quota = NegotiatorEditDynamicQuota(app, self)

class NegotiatorSelector(ObjectSelector):
    def __init__(self, app, name, pool):
        cls = app.model.com_redhat_grid.Negotiator

        super(NegotiatorSelector, self).__init__(app, name, cls)

        self.pool = pool

        self.add_filter(self.pool, cls.Pool)

        frame = "main.grid.pool.negotiator"
        col = ObjectLinkColumn(app, "name", cls.Name, cls._id, frame)
        self.add_column(col)

        self.add_attribute_column(cls.Machine)
        self.add_attribute_column(cls.System)

        #self.start = DaemonSelectionStart(app, self, "NEGOTIATOR")
        #self.stop = DaemonSelectionStop(app, self, "NEGOTIATOR")

class QmfGroupColumn(ItemTableColumn):
    def __init__(self, app, name, getter, negotiator, task):
        super(QmfGroupColumn, self).__init__(app, name)

        self.header_class = StaticTableColumnHeader

        self.title = None
        self.getter = getter
        #self.align = "right"
        self.user = False
        self.negotiator = negotiator
        self.task = task
        self.user_task = None

    def render_title(self, session, *args):
        return self.title

    def render_content(self, session, group):
        data = self.getter(session)
        for x in data:
            if x[0] == group:
                return self.render_data(session, x)

        return ""

    def render_data(self, session, data):
        href = self.task.get_href(session)
        content = data[2][2] and str(data[1]) or "NOT SET"
        return fmt_link(href, content, "", "", self.fmt_hover(data[0]))

    def fmt_hover(self, group):
        return "Edit the %s" % self.title

class NegotiatorOverview(ItemTable):
    def __init__(self, app, name, negotiator, frame):
        super(NegotiatorOverview, self).__init__(app, name)

        self.update_enabled = True
        #self.defer_enabled = True

        self.group_helper = GroupHelper(app, "groups", negotiator)
        self.add_child(self.group_helper)

        self.expand = Parameter(app, "expand")
        self.add_parameter(self.expand)

        col = self.ExpandColumn(app, "expand_col", self.expand)
        col.header_class = ItemTableColumnHeader
        col.width = "20px"
        self.add_column(col)

        col = self.GroupColumn(app, "group")
        self.add_column(col)
        self.set_default_column(col)

        task = frame.edit_dynamic_quota
        col = self.DynamicColumn(app, "dynamic", None, negotiator, task)
        col.title = "Dynamic Quota"
        self.add_column(col)

        col = self.SpacerColumn(app, "spacer")
        self.add_column(col)

        self.error_tmpl = WidgetTemplate(self, "error_html")
        self.loading_tmpl = WidgetTemplate(self, "deferred_html")

    def render_title(self, session):
        return "Quotas"

    def render_class(self, session):
        return "class=\"mobjects\""

    def render_deferred_content(self, session):
        return "Loading..."

    def do_render(self, session):
        tmpl = self.error_tmpl
        msg = None

        try:
            info = self.group_helper.get_config_info(session)
        except AssertionError:
            msg = "Missing negotiator"
        except Exception, e:
            msg = e.message
            if msg == "Loading":
                tmpl = self.loading_tmpl
            else:
                log.exception(e)
        else:
            return super(NegotiatorOverview, self).do_render(session)

        writer = Writer()
        if "is unknown" in msg:
            msg = "Please try again later"
        err_msg = "The data is not available at this time. <br/>%s" % msg
        tmpl.render(writer, session, err_msg)
        return writer.to_string()

    def render_error_msg(self, session, msg):
        return msg

    def do_get_items(self, session):
        info = self.group_helper.get_config_info(session)

        names = list(info)
        names = sorted(names)
        if self.reversed.get(session):
            names = reversed(names)

        expanded = self.expand.get(session)

        # make a list of groups to show based on what is expanded
        parent = expanded and self.group_helper.get_parent(session, expanded) or None
        items = list()
        for name in names:
            # always show top level groups
            if "." not in name:
                items.append(name)
            # add the current expanded item
            elif expanded == name:
                items.append(name)
            # add the ancestors
            elif expanded and expanded.startswith(name+"."):
                items.append(name)
            # add the direct children of expanded item
            elif expanded and name.startswith(expanded+"."):
                sub_name = name[len(expanded) + 1:]
                if "." not in sub_name:
                    items.append(name)
            # and direct children of expanded item's parent
            elif parent and name.startswith(parent+"."):
                sub_name = name[len(parent) + 1:]
                if "." not in sub_name:
                    items.append(name)

        self.group_helper.get_config_for_groups(session, "GROUP_QUOTA_DYNAMIC", items)

        return items

    class ExpandColumn(ItemTableColumn):
        def __init__(self, app, name, expand):
            super(NegotiatorOverview.ExpandColumn, self).__init__(app, name)

            self.expand = expand

        def render_title(self, session, *args):
            return ""

        def get_class_list(self, session):
            return ["expand"]

        def render_content(self, session, group):
            if not self.parent.group_helper.has_child(session, group):
                return ""

            info = self.parent.group_helper.get_config_info(session)
            # return a " or - depending on current expand
            expand = self.expand.get(session)
            if expand and expand.startswith(group):
                state = "-"
            else:
                state = "+"

            parent = self.parent.group_helper.get_parent(session, group)
            return self.render_item_link(session, group, parent, state)

        def render_item_link(self, session, group, parent, state):
            branch = session.branch()

            next_expand = state == "+" and group or parent
            self.expand.set(branch, next_expand)

            hover = state == "-" and "Collapse" or "Expand"
            cls = "action"

            return fmt_link(branch.marshal(), state, class_=cls, link_title=hover)

    class GroupColumn(NonSortableTableColumn):
        def render_title(self, session, *args):
            return "Name"

        def render_content(self, session, group):
            # if a user
            if "." in group:
                parts = group.split('.')
                indent = len(parts) - 1
                user = parts[indent]
                return "<span style='padding-left: %dem;'>%s</span>" % (indent, user)

            return group

    class DynamicColumn(QmfGroupColumn):
        def render_content(self, session, group):
            value = self.parent.group_helper.get_config_value(session, group, "GROUP_QUOTA_DYNAMIC")
            self.task.form.group_leader.set(session, group)

            try:
                fval = float(value)
            except ValueError:
                return value
            except TypeError:
                if isinstance(value, Exception):
                    content = "<span class='QuotaError'>error</span>"
                    return fmt_link("#", content, "", "", value.message)

            href = self.task.get_href(session)
            content = "%s%%" % str(round(fval * 100.0, 2))
            return fmt_link(href, content, "", "", self.fmt_hover(""))

    class SpacerColumn(ItemTableColumn):
        def render_title(self, session, *args):
            return ""

        def render_content(self, session, _):
            return ""

class GroupHelper(Widget):
    def __init__(self, app, name, negotiator):
        super(GroupHelper, self).__init__(app, name)

        self.negotiator = negotiator

        self.info = Attribute(app, "info")
        self.add_attribute(self.info)

        self.loading = Attribute(app, "loading")
        self.add_attribute(self.loading)

    def get_config_info(self, session):
        negotiator = self.negotiator.get(session)

        info = self.info.get(session)

        if not info:
            results = self.app.model.get_negotiator_group_names(negotiator)
            groups = results.data

            if not groups:
                if (not results.status) and (not results.exception):
                    raise QmfException("Loading")
                    #results.exception = QmfException("Request timed out. Please refresh the page to try again.")
            if results.exception:
                raise results.exception
            else:
                try:
                    groups = self.split_group_names(groups)
                except Exception, e:
                    log.exception(e)
                    groups = []

                info = dict()
                for group in groups:
                    info[group] = dict()

            self.info.set(session, info)

        return info

    def has_child(self, session, group):
        negotiator = self.negotiator.get(session)
        info = self.get_config_info(session)

        try:
            return info[group]['has_child']
        except KeyError:
            info[group]['has_child'] = False
            for key in info:
                if key.startswith(group+"."):
                    info[group]['has_child'] = True
                    break

        self.info.set(session, info)
        return info[group]['has_child']

    def get_parent(self, session, group):
        negotiator = self.negotiator.get(session)
        info = self.get_config_info(session)

        try:
            return info[group]['parent']
        except KeyError:
            parent = ""
            for key in info:
                if key != group and group.startswith(key):
                    if len(key) > len(parent):
                        parent = key

            info[group]['parent'] = parent

        self.info.set(session, info)
        return info[group]['parent']

    def get_siblings(self, session, node):
        negotiator = self.negotiator.get(session)
        info = self.get_config_info(session)

        siblings = list()
        (ng, s, nn) = rpartition(node, ".")
        for group in info:
            (g, s, n) = rpartition(group, ".")
            if g == ng:
                siblings.append(group)

        return siblings

    def get_group_names(self, session):
        info = self.get_config_info(session)
        return list(info)

    def split_group_names(self, group_string):
        groups = []
        g_string = group_string.replace(", ", ",")
        return  g_string.split(",")

    def get_config_for_groups(self, session, config, groups):
        info = self.get_config_info(session)

        needed_groups = [x for x in groups if not config in info[x]]

        if len(needed_groups) > 0:
            negotiator = self.negotiator.get(session)

            results = self.app.model.get_negotiator_config_values(negotiator, needed_groups, config)
            raw_configs = results.data

            try:
                for config in raw_configs:
                    for group in raw_configs[config]:
                        if raw_configs[config][group].error:
                            info[group][config] = raw_configs[config][group].error
                        else:
                            info[group][config] = raw_configs[config][group].data['Value']
            except Exception, e:
                log.exception(e)

            self.info.set(session, info)
        return info

    def get_config_value(self, session, group, config):
        info = self.get_config_info(session)
        try:
            return info[group][config]
        except KeyError:
            try:
                info = self.get_config_for_groups(session, config, [group])
                return info[group][config]
            except:
                self.loading.set(session, "loading")
                return "<em>loading</em>"

    def get_unclaimed_dyn_quota(self, session, groups):
        info = self.get_config_info(session)
        total = 0.0
        for group in groups:
            value = info[group]["GROUP_QUOTA_DYNAMIC"]
            try:
                total = total + float(value)
            except:
                pass

        val = 1.0 - total
        val = max(0, val)
        val = min(1.0, val)
        return val

class GroupAddForm(ObjectFrameTaskForm):
    def __init__(self, app, name, task):
        super(GroupAddForm, self).__init__(app, name, task)

        self.defer_enabled = True

        self.group_helper = GroupHelper(app, "groups", self.object)
        self.add_child(self.group_helper)

        self.group_name = NameField(app, "name")
        self.add_field(self.group_name)

    def render_title(self, session):
        return "Add Group"

    def process_submit(self, session):
        negotiator = self.object.get(session)
        group_name = self.group_name.get(session)
        original_groups = self.group_helper.get_group_names(session)

        if self.is_valid(group_name):
            if group_name not in original_groups:
                original_groups.append(group_name)
                new_groups = ", ".join(original_groups)
                self.task.invoke(session, negotiator, "GROUP_NAMES", new_groups)
                self.task.reconfig(negotiator)

                info = self.group_helper.get_config_info(session)
                info[group_name] = dict()

        self.task.exit_with_redirect(session)

    def is_valid(self, group):
        ret = False
        if len(group) and not "." in group and not " " in group:
            ret = True
        return ret

class GroupForm(ObjectFrameTaskForm):
    def __init__(self, app, name, task):
        super(GroupForm, self).__init__(app, name, task)

        cls = app.model.com_redhat_grid.Negotiator
        self.negotiator = RosemaryObjectParameter(app, "neg", cls)
        self.add_parameter(self.negotiator)

        self.group_helper = GroupHelper(app, "groups", self.negotiator)
        self.add_child(self.group_helper)

        group_name = Parameter(app, "gn")
        self.add_parameter(group_name)

        self.group_names = ListParameter(app, "group_names", group_name)
        self.add_parameter(self.group_names)

        original_value = Parameter(app, "ov")
        self.add_parameter(original_value)

        self.original_values = ListParameter(app, "original_values", original_value)
        self.add_parameter(self.original_values)

        self.defer_enabled = True

    def render_group_name(self, session, group):
        return group

    def render_group_name_path(self, session, group):
        return self.group_names.path

    def render_original_value_path(self, session, group):
        return self.original_values.path

    def render_form_class(self, session):
        return ""

class EditPrioFactorForm(GroupForm):
    def __init__(self, app, name, task):
        super(EditPrioFactorForm, self).__init__(app, name, task)

        self.field_tmpl = WidgetTemplate(self, "field_html")

        factor = Parameter(app, "factor")
        self.add_parameter(factor)

        self.factors = ListParameter(app, "factors", factor)
        self.add_parameter(self.factors)

    def render_title(self, session):
        return "Edit Group Priority Factor"

    def render_form_class(self, session):
        return "PrioFactorForm"

    def render_data_col_header(self, session):
        return "Priority Factor"

    def render_chart(self, session):
        return None

    def render_groups(self, session):
        writer = Writer()

        groups = self.group_helper.get_priority_factor(session)
        for group in groups:
            self.field_tmpl.render(writer, session, group)

        return writer.to_string()

    def render_factor_name(self, session, group):
        return self.factors.path

    def render_factor_value(self, session, group):
        return group[2][2] and group[1] or "--NOT SET--"

    def process_submit(self, session):
        negotiator = self.object.get(session)
        factors = self.factors.get(session)
        group_names = self.group_names.get(session)
        original_values = self.original_values.get(session)

        changed = False
        for group, new_value, original_value in zip(group_names, factors, original_values):
            if new_value != original_value:
                if self.is_valid_factor(new_value):
                    self.task.invoke(session, negotiator, "GROUP_PRIO_FACTOR_"+group, new_value)
                    changed = True
        if changed:
            self.task.reconfig(negotiator)
        self.task.exit_with_redirect(session)

    def is_valid_factor(self, value):
        try:
            factor = float(value)
        except:
            return False
        return factor >= 1.0

class UserPrioFactorForm(EditPrioFactorForm):
    def __init__(self, app, name, task):
        super(UserPrioFactorForm, self).__init__(app, name, task)

        self.group = Parameter(app, "group")
        self.add_parameter(self.group)

    def render_title(self, session):
        return "Edit User Priority Factor"

    def render_groups(self, session):
        writer = Writer()

        group = self.group.get(session)
        users = list(self.group_helper.get_priority_factor(session, [group], True))
        for user in users:
            self.field_tmpl.render(writer, session, user)

        return writer.to_string()

    def render_form_class(self, session):
        return "UserPrioFactorForm"

    def process_submit(self, session):
        negotiator = self.object.get(session)
        factors = self.factors.get(session)
        group = self.group.get(session)
        original_values = self.original_values.get(session)

        changed = False
        for user, new_value, original_value in zip([group], factors, original_values):
            if new_value != original_value:
                if self.is_valid_factor(new_value):
                    self.task.invoke(session, negotiator, "GROUP_PRIO_FACTOR_"+user, new_value)
                    changed = True
        if changed:
            self.task.reconfig(negotiator)
        self.task.exit_with_redirect(session)

class EditDynamicQuotaForm(GroupForm):
    def __init__(self, app, name, task):
        super(EditDynamicQuotaForm, self).__init__(app, name, task)

        self.field_tmpl = WidgetTemplate(self, "field_html")
        self.unclaimed_tmpl = WidgetTemplate(self, "unclaimed_html")

        quota = Parameter(app, "quota")
        self.add_parameter(quota)

        self.quotas = ListParameter(app, "quotas", quota)
        self.add_parameter(self.quotas)

        self.group_leader = Parameter(app, "group_leader")
        self.add_parameter(self.group_leader)

        self.chart = PriorityPieChart \
            (app, "chart", self.object, self.group_helper, self.group_leader)
        self.add_child(self.chart)

    def render_title(self, session):
        return "Edit Dynamic Group Quota"

    def render_submit_content(self, session):
        return "Change percentage"

    def render_form_class(self, session):
        return "priorityForm"

    def render_data_col_header(self, session):
        return "Percent"

    def render_groups(self, session):
        writer = Writer()

        group_leader = self.group_leader.get(session)
        groups = self.group_helper.get_siblings(session, group_leader)
        for group in sorted(groups):
            self.field_tmpl.render(writer, session, group)

        self.unclaimed_tmpl.render(writer, session, "Unclaimed")

        return writer.to_string()

    def render_quota_name(self, session, group):
        return self.quotas.path

    def render_quota_value(self, session, group):
        value = self.group_helper.get_config_value(session, group, "GROUP_QUOTA_DYNAMIC")
        try:
            return round(float(value) * 100.0, 2)
        except:
            return 0.0

    def render_unclaimed_value(self, session, group):
        group_leader = self.group_leader.get(session)
        groups = self.group_helper.get_siblings(session, group_leader)
        unclaimed = self.group_helper.get_unclaimed_dyn_quota \
            (session, groups)
        return round(float(unclaimed) * 100.0, 2)

    def render_chart_id(self, session):
        return self.chart.render_id(session)

    def process_submit(self, session):
        negotiator = self.negotiator.get(session)
        quotas = self.quotas.get(session)
        group_names = self.group_names.get(session)
        original_values = self.original_values.get(session)
        info = self.group_helper.get_config_info(session)

        changed = False
        for group, new_value, original_value in zip(group_names, quotas, original_values):
            if group == "Unclaimed":
                continue
            quota = self.check_quota(new_value, original_value)
            if quota:
                self.task.invoke(session, negotiator,
                                 "GROUP_QUOTA_DYNAMIC_" + group, quota)
                info[group]["GROUP_QUOTA_DYNAMIC"] = quota
                changed = True

        if changed:
            self.task.reconfig(negotiator)
            self.app.model.update_negotiator_config_value(negotiator)

        self.task.exit_with_redirect(session)

    def check_quota(self, quota, original):
        try:
            pri = float(quota)
        except:
            return None
        if pri < 0.0 or pri > 100.0:
            return '0'

        try:
            original = float(original)
        except:
            original = 0
        if pri == original:
            return None

        return str(pri / 100)

class EditStaticQuotaForm(GroupForm):
    def __init__(self, app, name, task):
        super(EditStaticQuotaForm, self).__init__(app, name, task)

        self.field_tmpl = WidgetTemplate(self, "field_html")

        quota = Parameter(app, "quota")
        self.add_parameter(quota)

        self.quotas = ListParameter(app, "quotas", quota)
        self.add_parameter(self.quotas)

    def render_title(self, session):
        return "Edit Static Group Quota"

    def render_submit_content(self, session):
        return "Change quota"

    def render_form_class(self, session):
        return "StaticQuotaForm"

    def render_data_col_header(self, session):
        return "Slots"

    def render_chart(self, session):
        return None

    def render_groups(self, session):
        writer = Writer()

        groups = self.group_helper.get_static_quota(session)
        for group in groups:
            self.field_tmpl.render(writer, session, group)

        return writer.to_string()

    def render_quota_name(self, session, group):
        return self.quotas.path

    def render_quota_value(self, session, group):
        return group[1]

    def process_submit(self, session):
        negotiator = self.object.get(session)
        quotas = self.quotas.get(session)
        group_names = self.group_names.get(session)
        original_values = self.original_values.get(session)

        changed = False
        for group, new_value, original_value in zip(group_names, quotas, original_values):
            if new_value != original_value:
                self.task.invoke(session, negotiator, "GROUP_QUOTA_"+group, new_value)
                changed = True
        if changed:
            self.task.reconfig(negotiator)
        self.task.exit_with_redirect(session)

class EditRegroupForm(GroupForm):
    def __init__(self, app, name, task):
        super(EditRegroupForm, self).__init__(app, name, task)

        self.field_tmpl = WidgetTemplate(self, "field_html")

        self.regroup = DictParameter(app, "regroup")
        self.add_parameter(self.regroup)

        original_regroup = Parameter(app, "or")
        self.add_parameter(original_regroup)

        self.original_regroups = ListParameter(app, "original_regroups", original_regroup)
        self.add_parameter(self.original_regroups)

    def render_title(self, session):
        return "Edit Autoregroup"

    def render_form_class(self, session):
        return "StaticQuotaForm"

    def render_data_col_header(self, session):
        return "Autoregroup"

    def render_chart(self, session):
        return None

    def render_groups(self, session):
        writer = Writer()

        regroups = list(self.group_helper.get_regroups(session))
        autoregroup = self.group_helper.get_autoregroup(session)
        autoregroup[0][0] = "Default"
        regroups.append(autoregroup[0])
        for regroup in regroups:
            self.field_tmpl.render(writer, session, regroup)

        return writer.to_string()

    def render_regroup_selected_TRUE(self, session, regroup):
        if regroup[2][2]:
            return (regroup[1] == 'TRUE') and "checked=\"checked\"" or ""

    def render_regroup_selected_FALSE(self, session, regroup):
        if regroup[2][2]:
            return (regroup[1] == 'FALSE') and "checked=\"checked\"" or ""

    def render_regroup_selected_UNDEFINED(self, session, regroup):
        if not regroup[2][2]:
            return "checked=\"checked\""

    def render_regroup_path(self, session, regroup):
        return self.regroup.get_instance_key(regroup[0])

    def render_original_regroup_path(self, session, regroup):
        return self.original_regroups.path

    def render_regroup_value(self, session, regroup):
        return regroup[2][2] and regroup[1] or "UNDEFINED"

    def process_submit(self, session):
        negotiator = self.object.get(session)
        regroup = self.regroup.get(session)
        group_names = self.group_names.get(session)
        original_regroups = self.original_regroups.get(session)

        changed = False
        for group, original_value in zip(group_names, original_regroups):
            if regroup[group] != original_value:
                if group == "Default":
                    self.task.invoke(session, negotiator, "AUTOREGROUP", regroup[group])
                else:
                    self.task.invoke(session, negotiator, "GROUP_AUTOREGROUP_"+group, regroup[group])
                changed = True
        if changed:
            self.task.reconfig(negotiator)
        self.task.exit_with_redirect(session)

class UserRegroupForm(EditRegroupForm):
    def __init__(self, app, name, task):
        super(UserRegroupForm, self).__init__(app, name, task)

        self.group = Parameter(app, "group")
        self.add_parameter(self.group)

    def render_groups(self, session):
        writer = Writer()

        group = self.group.get(session)
        regroups = list(self.group_helper.get_regroups(session, [group], True))
        for regroup in regroups:
            self.field_tmpl.render(writer, session, regroup)

        return writer.to_string()

    def render_form_class(self, session):
        return "UserRegroupForm"

    def process_submit(self, session):
        negotiator = self.object.get(session)
        regroup = self.regroup.get(session)
        group = self.group.get(session)
        original_regroups = self.original_regroups.get(session)

        changed = False
        for user, original_value in zip([group], original_regroups):
            if regroup[user] != original_value:
                self.task.invoke(session, negotiator, "GROUP_AUTOREGROUP_"+user, regroup[user])
                changed = True
        if changed:
            self.task.reconfig(negotiator)
        self.task.exit_with_redirect(session)

class PriorityPieChart(StatFlashChart):
    def __init__(self, app, name, negotiator, group_helper, group_leader):
        super(PriorityPieChart, self).__init__(app, name, negotiator)

        self.negotiator = negotiator

        self.chart_type = "pie"
        self.fullpageable = False
        self.update_enabled = False

        self.group_helper = group_helper
        self.group_leader = group_leader

    def render_title(self, session):
        pass

    def render_duration(self, session):
        pass

    def render_width(self, session):
        return 360

    def render_height(self, session):
        return 210

    def get_href_params(self, session):
        params = super(PriorityPieChart, self).get_href_params(session)
        params.append("width=%i" % self.render_width(session))
        params.append("height=%i" % self.render_height(session))

        # send the group names and values to the chart page so we
        # don't have to get them again
        group_leader = self.group_leader.get(session)
        groups = self.group_helper.get_siblings(session, group_leader)
        names = ["name=%s" % x for x in groups]
        params.extend(names)
        params.append("name=Unclaimed")

        info = self.group_helper.get_config_info(session)
        vals = ["value=%s" % info[x]["GROUP_QUOTA_DYNAMIC"] for x in groups]
        params.extend(vals)
        unclaimed = self.group_helper.get_unclaimed_dyn_quota(session, groups)
        params.append("value=%s" % str(unclaimed))

        return params

class NegotiatorGroupTask(ObjectFrameTask):
    def do_exit(self, session):
        self.app.main_page.main.grid.pool.view.show(session)

    def do_invoke(self, invoc, negotiator, group, value):
        # don't call self.qmf_call here since we need to
        # wait for the response before making the next QMF call
        action = QmfCall(self.app)
        result = action.execute(negotiator, "SetRawConfig", group, value)
        if result.error:
            raise result.error

        invoc.status_code = invoc.status
        try:
            group = group.split("_")[-1]
            invoc.description = "Set %s to %s" % (group, value)
        except:
            pass

        invoc.end()

    def reconfig(self, negotiator):
        action = QmfCall(self.app)
        action.execute(negotiator, "Reconfig")

class NegotiatorGroupAdd(NegotiatorGroupTask):
    def __init__(self, app, frame):
        super(NegotiatorGroupAdd, self).__init__(app, frame)

        self.form = GroupAddForm(app, self.name, self)

    def get_title(self, session):
        return ""

    def get_description(self, session):
        return "Add group"

class NegotiatorEditRegroup(NegotiatorGroupTask):
    def __init__(self, app, frame):
        super(NegotiatorEditRegroup, self).__init__(app, frame)

        self.form = EditRegroupForm(app, self.name, self)

    def get_title(self, session):
        return "Edit autoregroup"

class NegotiatorEditPrioFactor(NegotiatorGroupTask):
    def __init__(self, app, frame):
        super(NegotiatorEditPrioFactor, self).__init__(app, frame)

        self.form = EditPrioFactorForm(app, self.name, self)

    def get_title(self, session):
        return "Edit priority factor"

class NegotiatorEditDynamicQuota(NegotiatorGroupTask):
    def __init__(self, app, frame):
        super(NegotiatorEditDynamicQuota, self).__init__(app, frame)

        self.form = EditDynamicQuotaForm(app, self.name, self)

    def get_title(self, session):
        return ""

    def get_description(self, session):
        return "Edit dynamic quota"

    def do_enter(self, session, osession):
        group_leader = self.form.group_leader.get(osession)
        self.form.group_leader.set(session, group_leader)

        negotiator = self.frame.negotiator_attribute.get(osession)
        self.form.negotiator.set(session, negotiator)

class NegotiatorEditStaticQuota(NegotiatorGroupTask):
    def __init__(self, app, frame):
        super(NegotiatorEditStaticQuota, self).__init__(app, frame)

        self.form = EditStaticQuotaForm(app, self.name, self)

    def get_title(self, session):
        return "Edit static quota"

class NegotiatorUserTask(NegotiatorGroupTask):
    def do_enter(self, session, osession):
        #try:
        #    negotiator, group = ng
        #except:
        #    raise Exception("Must supply group for user forms")
        super(NegotiatorUserTask, self).do_enter(session, osession)
        #self.form.group.set(session, group)

class NegotiatorUserPrioFactor(NegotiatorUserTask):
    def __init__(self, app, frame):
        super(NegotiatorUserPrioFactor, self).__init__(app, frame)

        self.form = UserPrioFactorForm(app, self.name, self)

    def get_title(self, session):
        return "Edit user priority factor"

class NegotiatorUserRegroup(NegotiatorUserTask):
    def __init__(self, app, frame):
        super(NegotiatorUserRegroup, self).__init__(app, frame)

        self.form = UserRegroupForm(app, self.name, self)

    def get_title(self, session):
        return "Edit user autoregroup"
