from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *

from cumin.objectframe import *
from cumin.objectselector import *
from cumin.formats import *
from cumin.model import *
from cumin.parameters import *
from cumin.stat import *
from cumin.util import *
from cumin.widgets import *

from daemon import *
from job import *
from submitter import *
from submission import *

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.grid.scheduler")

class SchedulerFrame(ObjectFrame):
    def __init__(self, app, name, pool):
        cls = app.model.com_redhat_grid.Scheduler

        super(SchedulerFrame, self).__init__(app, name, cls)

        self.submitter = SubmitterFrame(app, "submitter")
        self.add_mode(self.submitter)

        overview = SchedulerOverview(app, "overview", self.object)
        self.view.add_tab(overview)

        submissions = SchedulerPoolSubmissionSelector(app, "submissions", pool)
        self.view.add_tab(submissions)

        cls = submissions.cls
        # We are referencing the jobserverRef, but passing in the scheduler id.
        # This gets fixed up in SchedulerPoolSubmissionSelectorget_data_value
        submissions.add_reference_filter(self.object, cls.jobserverRef)

        submitters = SubmitterSelector(app, "submitters", self.object)
        self.view.add_tab(submitters)

        #self.start = DaemonStart(app, self, "SCHEDD")
        #self.stop = DaemonStop(app, self, "SCHEDD")

class SchedulerPoolSubmissionSelector(PoolSubmissionSelector):
    def get_data_values(self, session):
        values = super(SchedulerPoolSubmissionSelector, self).get_data_values(session)

        jobserver_key = self.cls.jobserverRef.name

        # could also use self.frame.id.get(session)
        scheduler_id = values[jobserver_key]

        #we need to find the _id of the jobServer whose _schedulerRef_id matches our scheduler
        cls = self.app.model.com_redhat_grid.JobServer
        job_server = cls.get_object(session.cursor, _schedulerRef_id=scheduler_id)

        # Awkward: if there is no jobserver for this scheduler, don't select any records.
        # An alternative is to create our own adapter and override the get_data method
        jobserver_id = job_server and job_server._id or -1

        values[jobserver_key] = jobserver_id
        return values

class SchedulerSelector(ObjectSelector):
    def __init__(self, app, name, pool):
        cls = app.model.com_redhat_grid.Scheduler

        super(SchedulerSelector, self).__init__(app, name, cls)

        self.pool = pool

        self.add_filter(self.pool, cls.Pool)

        frame = "main.grid.pool.scheduler"
        col = ObjectLinkColumn(app, "name", cls.Name, cls._id, frame)
        self.add_column(col)

        self.add_attribute_column(cls.NumUsers)
        self.add_attribute_column(cls.TotalIdleJobs)
        self.add_attribute_column(cls.TotalRunningJobs)
        self.add_attribute_column(cls.TotalHeldJobs)

        #self.start = DaemonSelectionStart(app, self, "SCHEDD")
        #self.stop = DaemonSelectionStop(app, self, "SCHEDD")

class SchedulerGeneralStatSet(StatSet):
    def __init__(self, app, name, object):
        super(SchedulerGeneralStatSet, self).__init__(app, name, object)

        self.attrs = ("NumUsers", "TotalRunningJobs",
                      "TotalIdleJobs", "TotalHeldJobs",
                      "TotalJobAds", "TotalRemovedJobs")

class SchedulerOverview(Widget):
    def __init__(self, app, name, scheduler):
        super(SchedulerOverview, self).__init__(app, name)

        stats = SchedulerGeneralStatSet(app, "general", scheduler)
        self.add_child(stats)

        chart = self.UsersChart(app, "users", scheduler)
        chart.stats = ("NumUsers",)
        chart.duration.param.default = "3600"
        self.add_child(chart)

        chart = self.JobsChart(app, "jobs", scheduler)
        chart.stats = ("TotalRunningJobs", "TotalIdleJobs", "TotalHeldJobs")
        chart.duration.param.default = "3600"
        self.add_child(chart)

    def render_title(self, session):
        return "Overview"

    class UsersChart(StatFlashChart):
        def render_title(self, session):
            return "Users"

    class JobsChart(StatFlashChart):
        def render_title(self, session):
            return "Jobs"
