from mint import *
from wooly import *
from wooly.widgets import *
from cumin.objectframe import *
from cumin.objectselector import *
from cumin.stat import *
from cumin.widgets import *
from cumin.parameters import *
from cumin.formats import *
from cumin.util import *
from cumin.grid.slot import SlotFrame, SlotMap, SlotMapPage

from wooly import Session
import main

strings = StringCatalog(__file__)

class SystemSelector(ObjectSelector):
    def __init__(self, app, name):
        cls = app.model.com_redhat_sesame.Sysimage

        super(SystemSelector, self).__init__(app, name, cls)

        frame = "main.inventory.system"
        col = ObjectLinkColumn(app, "name", cls.nodeName, cls._id, frame)
        self.add_column(col)

        self.add_attribute_column(cls.osName)
        self.add_attribute_column(cls.machine)
        self.add_attribute_column(cls.memFree)
        self.add_attribute_column(cls.loadAverage1Min)

class SystemFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_sesame.Sysimage

        super(SystemFrame, self).__init__(app, name, cls)

        self.icon_href = "resource?name=system-36.png"

        overview = SystemStats(app, "overview", self.object)
        self.view.add_tab(overview)

class SystemGeneralStatSet(StatSet):
    def __init__(self, app, name, object):
        super(SystemGeneralStatSet, self).__init__(app, name, object)

        self.attrs = ("memFree", "swapFree",
                       "loadAverage1Min", "loadAverage5Min",
                       "loadAverage10Min", "procTotal",
                       "procRunning")

class SystemStats(Widget):
    def __init__(self, app, name, system):
        super(SystemStats, self).__init__(app, name)

        self.add_child(SystemGeneralStatSet(app, "stats", system))

        chart = StatFlashChart(app, "freemem", system)
        chart.stats = ("memFree",)
        self.add_child(chart)

        chart = StatFlashChart(app, "loadavg", system)
        chart.stats = ("loadAverage1Min",)
        self.add_child(chart)

        slots = SystemSlotMap(app, "system_slot_map", system)
        self.add_child(slots)

        slot_vis = SystemSlotFlashVis(app, "system_slot_flash", system)
        self.add_child(slot_vis)

    def render_title(self, session):
        return "Overview"

    def render_slot_job_url(self, session):
        #job = Identifiable("XXX")
        #return self.page.main.grid.pool.job.get_href(session, job, None)
        return session.marshal() # XXX

class SystemSlotMap(SlotMap):
    def __init__(self, app, name, system):
        super(SystemSlotMap, self).__init__(app, name)

        self.system = system
        self.slot_clip_size = 280

    def do_process(self, session):
        super(SystemSlotMap, self).do_process(session)

        system = self.system.get(session)
        self.slots.add_where_expr(session, "\"System\" = '%s'", system.nodeName)

    def render_image_href(self, session):
        system = self.system.get(session)

        page = self.app.system_slots_page
        sess = Session(page)

        #page.sysimage.set(sess, system._id)
        page.sysimage.set(sess, system)

        return sess.marshal()

    def render_slot_url(self, session):
        return ""

    def get_scheduler_select(self, session):
        system = self.system.get(session)
        return "system='%s'" % system.nodeName

class SystemSlotFlashVis(SystemSlotMap):
    def render_slots_href(self, session):
        system = self.system.get(session)

        page = self.app.system_slots_page
        sess = Session(page)

        page.sysimage.set(sess, system)
        page.json.set(sess, "slots")
        page.groups.set(sess, [])

        return sess.marshal()

    def render_slot_chart_width(self, session):
        return 400

    def render_slot_chart_height(self, session):
        return 400

class OldSystemView(CuminView):
    def __init__(self, app, name, system):
        super(OldSystemView, self).__init__(app, name, system)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)

        self.tabs.add_tab(SystemStats(app, "stats", system))
        self.tabs.add_tab(SystemServices(app, "services", system))
        self.tabs.add_tab(CuminDetails(app, "details", system))

class SystemServices(ItemSet):
    def __init__(self, app, name, system):
        super(SystemServices, self).__init__(app, name)

        self.system = system

    def render_title(self, session):
        return "Services"

    def do_get_items(self, session):
        system = self.system.get(session)

        daemons = list()
        daemon_types = [Scheduler, Collector, Negotiator]
        sql = "system = '%s'" % system.nodeName

        for daemon in daemon_types:
            system_daemons = daemon.select(sql)
            for devil in system_daemons:
                try:
                    daemons.append(devil)
                except Exception, e:
                    pass

        # using a loop instead of an sql select with an outer join
        brokers = Broker.select()

        for broker in brokers:
            if broker.system.nodeName == system.nodeName:
                daemons.append(broker)

        return daemons

    def render_item_content(self, session, item):
        if isinstance(item, Broker):
            broker = Identifiable(item.id)
            href = self.page.main.messaging.broker.get_href(session, broker)
            return fmt_link(href, item.port) # XXX item.port is too lame
        else:
            pool = Pool(item.Pool)
            self.page.main.grid.pool.object.set(session, pool)
            #self.page.main.show_grid_tab(session)
            daemon = Identifiable(item.id)
            if isinstance(item, Collector):
                self.page.main.grid.pool.view.collectors.show(session)
                href = self.page.main.grid.pool.collector.get_href \
                    (session, daemon)
            elif isinstance(item, Scheduler):
                self.page.main.grid.pool.view.schedulers.show(session)
                href = self.page.main.grid.pool.scheduler.get_href \
                    (session, daemon)
            elif isinstance(item, Negotiator):
                self.page.main.grid.pool.view.negotiators.show(session)
                href = self.page.main.grid.pool.negotiator.get_href \
                    (session, daemon)
            return fmt_link(href, item.Name)

    def render_item_type(self, session, item):
        if isinstance(item, Collector):
            return "Collector"
        elif isinstance(item, Scheduler):
            return "Scheduler"
        elif isinstance(item, Negotiator):
            return "Negotiator"
        elif isinstance(item, Broker):
            return "Broker"
        else:
            return "Daemon"

class SystemSlotMapPage(SlotMapPage):
    def __init__(self, app, name):
        self.cls = app.model.com_redhat_sesame.Sysimage
        self.sysimage = RosemaryObjectParameter(app, "id", self.cls)

        super(SystemSlotMapPage, self).__init__(app, name, self.sysimage, "System")

        self.add_parameter(self.sysimage)

    def do_process(self, session):
        super(SystemSlotMapPage, self).do_process(session)

        sysimage = self.sysimage.get(session)
        self.slots.add_where_expr(session, "\"System\" = '%s'", sysimage.nodeName)
