from rosemary.sqlquery import *
from wooly.datatable import *

from util import *

class QmfAdapter(DataAdapter):
    def __init__(self, app, method):
        super(QmfAdapter, self).__init__()

        self.app = app
        self.method = method
        self.default = dict()
        self.columns = list()

    def get_count(self, values):
        obj = values['obj']
        args = values['args']

        action = QmfCall(self.app)
        results = action.execute(obj, self.method, args)

        return results and len(results) or 0

    def get_data(self, values, options):
        obj = values['obj']
        args = values['args']

        action = QmfCall(self.app)
        results = action.execute(obj, self.method, args)

        if results.error:
            results.data = self.default

        rows = self.process_results(results)
        return rows[options.offset:options.offset + options.limit]

    def process_results(self, results):
        """ take the dict response from the qmf call and return a list of lists """

        records = list()

        if results:
            for key in results:
                row = self.process_record(key, results[key])
                records.append(row)

        return records

    def process_record(self, key, record):
        field_data = list()
        for column in self.columns:
            try:
                val = record[column.name]
            except KeyError:
                val = 0
            field_data.append(val)
        return field_data

class ObjectQmfAdapter(QmfAdapter):
    def __init__(self, app, cls):
        super(ObjectQmfAdapter, self).__init__(app, cls._name)

        self.cls = cls

        self.fields_by_attr = dict()

class QmfField(DataAdapterField):
    def __init__(self, adapter, column):
        python_type = column.type.python_type

        super(QmfField, self).__init__(adapter, column.name, python_type)

        self.column = column

        self.adapter.columns.append(column)


class ObjectQmfField(QmfField):
    def __init__(self, adapter, attr):
        assert isinstance(adapter, ObjectQmfAdapter), adapter

        super(ObjectQmfField, self).__init__(adapter, attr.sql_column)

        self.attr = attr

        self.adapter.fields_by_attr[self.attr] = self

    def get_title(self, session):
        return self.attr.title or self.attr.name
