import logging

from ConfigParser import *

log = logging.getLogger("parsley.config")

class Config(object):
    def __init__(self):
        self.sections = list()

    def parse_files(self, paths):
        parser = SafeConfigParser()
        found = parser.read(paths)

        for path in found:
            log.info("Read config file '%s'", path)

        if not found:
            log.info("No config files found at %s", ", ".join(paths))

        sections = ConfigValues()

        for section in self.sections:
            try:
                values = section.parse(parser)
            except NoSectionError:
                continue

            sections[section.name] = values

        return sections

class ConfigSection(object):
    def __init__(self, config, name):
        assert isinstance(config, Config)

        self.config = config
        self.config.sections.append(self)

        self.name = name

        self.parameters = list()

    def parse(self, parser):
        values = ConfigValues()

        for param in self.parameters:
            name = param.name.replace("-", "_")
            string = None

            try:
                string = parser.get(self.name, param.name)
            except NoOptionError:
                try:
                    string = parser.get("common", param.name)
                except NoOptionError:
                    pass

            if string is None:
                value = param.default
            else:
                value = param.unmarshal(string)

            values[name] = value

        return values

class ConfigParameter(object):
    def __init__(self, section, name, type):
        assert isinstance(section, ConfigSection)

        self.section = section
        self.section.parameters.append(self)

        self.name = name
        self.type = type

        self.default = None

    def unmarshal(self, string):
        return self.type(string)

class ConfigValues(dict):
    def __getattr__(self, name):
        return self[name]
