import os
import wooly

class StringCatalog(object):
    def __init__(self, file):
        self.strings = None
        self.path = os.path.splitext(file)[0] + ".strings"

    def clear(self):
        self.strings = None

    def load(self):
        file = open(self.path)
        try:
            self.strings = parse_catalog_file(file)
        finally:
            file.close()

    def get(self, key):
        if not self.strings:
            self.load()

        return self.strings.get(key)

    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, self.path)

def parse_catalog_file(file):
    strings = dict()
    key = None
    writer = wooly.Writer()

    for line in file:
        line = line.rstrip()

        if line.startswith("[") and line.endswith("]"):
            if key:
                strings[key] = writer.to_string().strip()

            writer = wooly.Writer()

            key = line[1:-1]

            continue

        writer.write(line)
        writer.write("\r\n")

    strings[key] = writer.to_string().strip()

    return strings

class ResourceFinder(object):
    def __init__(self):
        self.dirs = list()
        self.whitelists = dict() # dir => (name1, name2, ...)

    def add_dir(self, dir):
        if not os.path.isdir(dir):
            return

        self.dirs.append(dir)
        self.whitelists[dir] = set(os.listdir(dir))

    def find(self, name):
        file = None

        for dir in self.dirs:
            whitelist = self.whitelists[dir]

            if name in whitelist:
                try:
                    path = os.path.join(dir, name)
                    file = open(path, "r")
                except IOError:
                    pass

        return file
